@@ -1,7 +1,7 @@
 # Copyright 2011-2022 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 """
-This is the :mod:`atelier` package.
+See :doc:`/index`.
 
 .. autosummary::
    :toctree:
@@ -23,7 +23,8 @@ from .setup_info import SETUP_INFO
 
 __version__ = SETUP_INFO['version']
 
-intersphinx_urls = dict(docs="https://lino-framework.gitlab.io/atelier")
+# intersphinx_urls = dict(docs="https://lino-framework.gitlab.io/atelier")
+intersphinx_urls = dict(docs="https://atelier.lino-framework.org")
 srcref_url = 'https://gitlab.com/lino-framework/atelier/blob/master/%s'
 # doc_trees = ['docs']
 
