@@ -22,9 +22,10 @@ from pathlib import Path
 import atelier
 
 
-def setup_from_tasks(
-    globals_dict, main_package=None,
-    django_settings_module=None, **kwargs):
+def setup_from_tasks(globals_dict,
+                     main_package=None,
+                     django_settings_module=None,
+                     **kwargs):
     """
     This is the function you must call from your :xfile:`tasks.py` file
     in order to activate the tasks defined by atelier.
@@ -47,7 +48,8 @@ def setup_from_tasks(
     if '__file__' not in globals_dict:
         raise Exception(
             "No '__file__' in %r. "
-            "First argument to setup_from_tasks() must be `globals()`." % globals_dict)
+            "First argument to setup_from_tasks() must be `globals()`." %
+            globals_dict)
 
     tasks_file = Path(globals_dict['__file__'])
     if not tasks_file.exists():
@@ -64,7 +66,8 @@ def setup_from_tasks(
     if kwargs:
         for k, v in kwargs.items():
             if k not in prj.config:
-                raise Exception("Invalid keyword argument '{}={}'.".format(k, v))
+                raise Exception("Invalid keyword argument '{}={}'.".format(
+                    k, v))
         prj.config.update(kwargs)
 
     if django_settings_module is not None:
