@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2013-2021 Rumma & Ko Ltd
+# Copyright 2013-2022 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 """
 This is the module that defines the invoke namespace.
@@ -8,14 +8,16 @@ It is imported by :func:`atelier.invlib.setup_from_tasks` which passes
 it to :func:`invoke.Collection.from_module`.
 """
 
+from os.path import commonprefix
 import os
 import sys
 import io
 import glob
 import time
 import datetime
-from datetime import timedelta
 import shutil
+from datetime import timedelta
+from importlib import import_module
 
 from atelier.utils import i2d, i2t
 from babel.dates import format_date
@@ -39,11 +41,13 @@ from .utils import must_confirm
 
 import requests
 
+
 def checkurl(url, name):
     r = requests.get(url, allow_redirects=False)
     if r.status_code != 200:
         print("{name} ({url}) returns {r.status_code}".format(**locals()))
 
+
 def add_slash(url):
     if not url.endswith('/'):
         url += "/"
@@ -111,9 +115,15 @@ def cleanup_pyc(p, batch=False):  # no longer used
     """Thanks to oddthinking on https://stackoverflow.com/questions/2528283
     """
     for root, dirs, files in os.walk(p):
-        pyc_files = [filename for filename in files if filename.endswith(".pyc")]
-        py_files = set([filename for filename in files if filename.endswith(".py")])
-        excess_pyc_files = [pyc_filename for pyc_filename in pyc_files if pyc_filename[:-1] not in py_files]
+        pyc_files = [
+            filename for filename in files if filename.endswith(".pyc")
+        ]
+        py_files = set(
+            [filename for filename in files if filename.endswith(".py")])
+        excess_pyc_files = [
+            pyc_filename for pyc_filename in pyc_files
+            if pyc_filename[:-1] not in py_files
+        ]
         for excess_pyc_file in excess_pyc_files:
             full_path = os.path.join(root, excess_pyc_file)
             if batch or confirm("Remove excess file %s:" % full_path):
@@ -141,8 +151,8 @@ def py_clean(ctx, batch=False):
         if p.exists():
             paths.append(p)
     if len(paths):
-        if batch or confirm(
-            "Remove {0} __pycache__ directories".format(len(paths))):
+        if batch or confirm("Remove {0} __pycache__ directories".format(
+                len(paths))):
             for p in paths:
                 rmtree_after_confirm(p, True)
 
@@ -176,13 +186,13 @@ def py_clean(ctx, batch=False):
         for p in glob.glob(os.path.join(ctx.root_dir, pat)):
             files.append(p)
     if len(files):
-        if batch or confirm(
-                "Remove {0} cleanable files".format(len(files))):
+        if batch or confirm("Remove {0} cleanable files".format(len(files))):
             for p in files:
                 os.remove(p)
 
 
 class RstFile(object):
+
     def __init__(self, local_root, url_root, parts):
         self.path = local_root / ('/'.join(parts) + '.rst')
         self.url = url_root + "/" + "/".join(parts) + '.html'
@@ -194,6 +204,7 @@ class RstFile(object):
 
 
 class MissingConfig(Exception):
+
     def __init__(self, name):
         msg = "Must set `config.{0}` in `tasks.py`!"
         msg = msg.format(name)
@@ -208,13 +219,22 @@ def update_fixtures(ctx):
     # m = import_module(ctx.fixtures_updater)
     # m.update_fixtures(ctx)
 
+
+@task(name='run')
+def run_script(ctx, script, after=None, start=None):
+    """Run a given python script on all the demo projects."""
+
+    cmd = f"manage.py run {Path(script).resolve()}"
+    run_in_demo_projects(ctx, cmd, after=after, start=start)
+
+
 @task(name='test')
 def run_tests(ctx):
     """Run the test suite of this project."""
     # assert os.environ['COVERAGE_PROCESS_START']
     cmd = ctx.test_command
     if cmd:
-        print("Run test command {0} :".format(cmd))
+        print("Run test command {0} in {1}:".format(cmd, ctx.root_dir))
         with ctx.cd(ctx.root_dir):
             ctx.run(cmd, pty=True)
 
@@ -231,7 +251,10 @@ def write_readme(ctx):
         return
     atelier.current_project.load_info()
     info = atelier.current_project.SETUP_INFO
-    if not info.get('long_description'):
+    long_description = info.get('long_description')
+    if not long_description:
+        return
+    if long_description.startswith("===="):  # e.g. in synodal
         return
     # if len(ctx.doc_trees) == 0:
     #     # when there are no docs, then the README file is manually maintained
@@ -248,11 +271,19 @@ def write_readme(ctx):
 
     title = rstgen.header(1, "The ``{}`` package".format(info['name']))
 
-    txt = """\
-{title}
+    do_not_edit = """
+..
+    This file has been generated by ``inv bd` from the docstring of the
+    :file:`setup.py` file. Do not edit this file, but edit the :file:`setup.py` file
+    and run ``doctest setup.py`` before generating it again.
 
+"""
+    do_not_edit = "\n"  # good idea, but lets do it later
+
+    txt = """\
+{title}{do_not_edit}
 {long_description}
-""".format(title=title, **info)
+""".format(title=title, do_not_edit=do_not_edit, **info)
     if readme.exists() and readme.read_text() == txt:
         return
     must_confirm("Overwrite %s" % readme.absolute())
@@ -274,7 +305,7 @@ def build_docs(ctx, only=None):
         ctx.run(cmd, pty=True)
     for tree in atelier.current_project.get_doc_trees():
         if tree.src_path:
-           if only is None or tree.rel_path == only:
+            if only is None or tree.rel_path == only:
                 tree.build_docs(ctx)
 
 
@@ -287,6 +318,14 @@ def clean(ctx, batch=False):
     # clean_demo_caches()
 
 
+@task(name='pepper')
+def pepper(ctx):
+    "Run PEP formatting and checking."
+    # NB The ""-e '*/help_texts.py'" can be removed soon because newly generated help_texts.py files now contain "# yapf: disable"
+    cmd = "yapf -rip -e '*/help_texts.py' -e '*.tpl.py' -e '*/static_root/*' -e '*/templates/*' ."
+    ctx.run(cmd, pty=True)
+
+
 @task(name='sdist')
 def setup_sdist(ctx):
     "Create a source distribution."
@@ -297,16 +336,18 @@ def setup_sdist(ctx):
     if not info.get('name'):
         return
     show_pypi_status(ctx, False)
-    dist_dir = ctx.sdist_dir.format(prj=info.get('name'))
+    dist_dir = str(ctx.sdist_dir).format(prj=info.get('name'))
     args = [sys.executable, "setup.py"]
     args += ["sdist", "--formats=gztar"]
     args += ["--dist-dir", dist_dir]
     ctx.run(' '.join(args), pty=True)
 
 
-@task(name='release', help={
-    'batch': "Don't ask for confirmations.",
-    'branch': "Create a version branch."})
+@task(name='release',
+      help={
+          'batch': "Don't ask for confirmations.",
+          'branch': "Create a version branch."
+      })
 def pypi_release(ctx, batch=False, branch=False):
     """
     Publish a new version to PyPI.
@@ -321,7 +362,7 @@ def pypi_release(ctx, batch=False, branch=False):
         return
     version = info['version']
     # dist_dir = Path(ctx.sdist_dir) / info['name']
-    dist_dir = ctx.sdist_dir.format(prj=info.get('name'))
+    dist_dir = str(ctx.sdist_dir).format(prj=info.get('name'))
     dist_dir += "/{name}-{version}.tar.gz".format(**info)
 
     show_revision_status(ctx)
@@ -382,13 +423,17 @@ def test_sdist(ctx):
             cmd = activate + ';' + cmd
             ctx.run(cmd, pty=True)
 
-        vrun("pip install --download {0} {1}".format(ctx.pypi_dir, info['name']))
+        vrun("pip install --download {0} {1}".format(ctx.pypi_dir,
+                                                     info['name']))
         # DEPRECATION: pip install --download has been deprecated and will be removed in the future. Pip now has a download command that should be used instead.
 
         # vrun("pip download {0}".format(info['name']))
 
-        vrun("pip install --no-allow-external --no-index --no-cache-dir -f {} -f {} {}".format(
-            ctx.sdist_dir, ctx.pypi_dir, info['name']))
+        dist_dir = str(ctx.sdist_dir).format(prj=info.get('name'))
+
+        vrun(
+            "pip install --no-allow-external --no-index --no-cache-dir -f {} -f {} {}"
+            .format(sdist_dir, ctx.pypi_dir, info['name']))
         # vrun("pip install -f {0} {1}".format(ctx.sdist_dir, info['name'])
 
         vrun("inv test")
@@ -510,11 +555,18 @@ def edit_blog_entry(ctx, today=None):
 
         entry.path.write_text(content)
         # touch the year's index for Sphinx:
-        set_times(entry.path.parent / 'index.rst')
-
         # Touch the main index.rst in case it contains a blogger_latest directive.
         # We assume that if there is one, it is there.
-        set_times(Path(ctx.blog_root) / 'index.rst')
+        indexes = []
+        indexes.append(entry.path.parent / 'index.rst')
+        indexes.append(entry.path.parent.parent / 'index.rst')
+        indexes.append(entry.path.parent.parent.parent / 'index.rst')
+        for i in indexes:
+            if i.exists():
+                set_times(i)
+
+        # set_times(entry.path.parent.parent / 'index.rst')
+        # set_times(Path(ctx.blog_root) / 'index.rst')
 
     args = [ctx.editor_command.format(entry.path)]
     args += [str(entry.path)]
@@ -525,6 +577,9 @@ def edit_blog_entry(ctx, today=None):
 @task(name='pd')
 def publish(ctx, only=None):
     """Publish docs. Upload docs to public web server. """
+    if not atelier.current_project.published:
+        print("Not publishing {}".format(atelier.current_project))
+        return
     if not ctx.docs_rsync_dest:
         raise MissingConfig("docs_rsync_dest")
 
@@ -558,11 +613,14 @@ def show_pypi_status(ctx, severe=True):
     assert name and version
 
     from xmlrpc.client import ServerProxy
+
     # thanks to https://github.com/pypa/warehouse/issues/8753
     class RateLimitedServerProxy(ServerProxy):
+
         def __getattr__(self, name):
             time.sleep(1)
             return super(RateLimitedServerProxy, self).__getattr__(name)
+
     client = RateLimitedServerProxy('https://pypi.python.org/pypi')
     released_versions = client.package_releases(name)
     if len(released_versions) == 0:
@@ -579,9 +637,8 @@ def show_pypi_status(ctx, severe=True):
             # lastrel['upload_time'] = dt.ISO()
             print(LASTREL_INFO % lastrel)
         if severe and version in released_versions:
-            raise Exit(
-                "ABORT: %(name)s %(version)s has already been "
-                "released." % info)
+            raise Exit("ABORT: %(name)s %(version)s has already been "
+                       "released." % info)
 
 
 def get_blog_entry(ctx, today, now=None):
@@ -665,7 +722,10 @@ def update_catalog_code(ctx):
             args = [sys.executable, "setup.py"]
             args += ["update_catalog"]
             args += ["--domain django"]
-            args += ["-o", str(ld / to_locale(loc) / 'LC_MESSAGES' / 'django.po')]
+            args += [
+                "-o",
+                str(ld / to_locale(loc) / 'LC_MESSAGES' / 'django.po')
+            ]
             args += ["-i", str(ld / "django.pot")]
             args += ["-l", to_locale(loc)]
             cmd = ' '.join(args)
@@ -677,6 +737,7 @@ def update_catalog_code(ctx):
 # def list_projects(ctx, *cmdline_args):
 #     """List your projects."""
 
+
 def git_projects():
     for prj in load_projects():
         prj.load_info()
@@ -688,12 +749,14 @@ def git_projects():
 def check_config(ctx):
     """Check configuration integrity."""
     from git import Repo
-    import giturlparse # pip install git-url-parse
+    import giturlparse  # pip install git-url-parse
     prj = atelier.current_project
     prj.load_info()
     if prj.config['revision_control_system'] != 'git':
         if (prj.root_dir / '.git').exists():
-            print("revision_control_system is not 'git', but .git directory exists")
+            print(
+                "revision_control_system is not 'git', but .git directory exists"
+            )
         return
     info = prj.SETUP_INFO
     if len(info) == 0:
@@ -736,9 +799,6 @@ def check_config(ctx):
             print("blogref_url ({}) returns {}".format(url, r.status_code))
 
 
-
-
-
 @task(name='ct')
 def commited_today(ctx, today=None):
     """Print all today's commits to stdout."""
@@ -750,9 +810,8 @@ def commited_today(ctx, today=None):
         ONEDAY = timedelta(days=1)
         yesterday = today - ONEDAY
         tomorrow = today + ONEDAY
-        list_options.update(
-            after=yesterday.strftime("%Y-%m-%d"),
-            before=tomorrow.strftime("%Y-%m-%d"))
+        list_options.update(after=yesterday.strftime("%Y-%m-%d"),
+                            before=tomorrow.strftime("%Y-%m-%d"))
     if False:
         list_options.update(max_count=5)
 
@@ -787,7 +846,8 @@ def commited_today(ctx, today=None):
 
         for c in it:
             # ts = time.strftime("%H:%M", time.gmtime(c.committed_date))
-            ts = time.strftime("%Y-%m-%d %H:%M", time.localtime(c.committed_date))
+            ts = time.strftime("%Y-%m-%d %H:%M",
+                               time.localtime(c.committed_date))
             rows.append([ts, desc, fmtcommit(c)])
 
     for p in git_projects():
@@ -806,15 +866,53 @@ def commited_today(ctx, today=None):
 #         with cd(p.root_dir):
 
 
-from importlib import import_module
-
-def run_in_demo_projects(ctx, py_cmd, cov=False, bare=False, out_stream=None):
+def run_in_demo_projects(ctx,
+                         py_cmd,
+                         cov=False,
+                         bare=False,
+                         after=None,
+                         start=None):
     """
     Run the given Python command line `py_cmd` in each demo project.
 
-    See also :attr:`ctx.demo_projects`.
+    When :attr:`bare` is True, return a string concatenated from the bare output
+    in each demo project.
+
+    See also :envvar:`demo_projects`.
     """
-    for dpname in ctx.demo_projects:
+    outputs = []
+    if after is None and start is None:
+        demo_projects = list(ctx.demo_projects)
+    else:
+        demo_projects = []
+        skipping = True
+        for dpname in ctx.demo_projects:
+            if after and dpname.endswith(after):
+                skipping = False
+                continue
+            if start and dpname.endswith(start):
+                skipping = False
+            if skipping:
+                continue
+            demo_projects.append(dpname)
+
+    basename = commonprefix(demo_projects)
+    # print("All demo projects start with", basename)
+    chop = len(basename)
+
+    if len(demo_projects) == 0:
+        return "" if bare else None
+    if bare:
+        print("Run `{}` in {} demo projects:".format(py_cmd,
+                                                     len(demo_projects)),
+              end=" ",
+              flush=True)
+    for dpname in demo_projects:
+        if bare:
+            print(dpname[chop:], end=" ", flush=True)
+        else:
+            print("-" * 80)
+            print("Run `{}` in {} :".format(py_cmd, dpname))
         dpmodule = import_module(dpname)
         pth = Path(dpmodule.__file__).parent
         # join each demo project to root_dir to avoid failure when
@@ -828,37 +926,56 @@ def run_in_demo_projects(ctx, py_cmd, cov=False, bare=False, out_stream=None):
                 os.environ['COVERAGE_FILE'] = datacovfile
             else:
                 cmd = sys.executable + ' ' + py_cmd
-            if not bare:
-                print("-" * 80)
-                print("Run in demo project {0}\n$ {1} :".format(dpname, cmd))
-            ctx.run(cmd, pty=True, out_stream=out_stream)
+            if bare:
+                out_stream = io.StringIO()
+                result = ctx.run(cmd, out_stream=out_stream)
+                if result.exited:
+                    print(out_stream.getvalue())
+                else:
+                    outputs.append(out_stream.getvalue())
+            else:
+                result = ctx.run(cmd, pty=True)
+            if result.exited is None:
+                raise Exception("Subprocess did not exit")
+            if result.exited:
+                raise Exception("Subprocess exited with {}".format(
+                    result.exited))
+    if bare:
+        print("DONE")
+        return "\n".join(outputs)
 
 
-@task(name='install', optional=['list', 'list_only'])
-def configure(ctx, list=False, list_only=False):
+@task(
+    name='install',
+    help=dict(
+        list="Don't install anything, just list the requirements to `stdout`.",
+        batch="Don't ask for confirmations."))
+def install(ctx, list=False, batch=False, after=None, start=None):
+    """
+    Install required Python packages to your Python environment and/or
+    to your `requirements-install.txt` file.
+
+    See https://atelier.lino-framework.org/invlib.html#command-inv-install
     """
 
-    Run `manage.py install` on every demo project. Takes two optional boolean
-    flags `--list` and `--list-only` and are mutually exclusive meaning cannot
-    be used both at the same time.
+    print("Collect list of required packages from all demo projects.")
+    cmd = 'manage.py install --list'
+    output = run_in_demo_projects(ctx,
+                                  cmd,
+                                  bare=True,
+                                  after=after,
+                                  start=start)
+    reqs = sorted(set(output.split()))
+    if list:
+        print("\n".join(reqs))
+        return
 
-    :cmd:`inv install --list` installs the required packages and also lists and
-    writes them into :file:`requirements-install.txt`.
+    cwd = os.getcwd()
 
-    :cmd:`inv install --list-only` does not install the packages and only lists
-    and writes them into :file:`requirements-install.txt`.
-
-    """
-
-    def do_listing():
-        print("Getting the list of required packages from all of the demo projects.")
-        cmd = 'manage.py install --list'
-        temp_f = io.StringIO()
-        run_in_demo_projects(ctx, cmd, bare=True, out_stream=temp_f)
-        temp_f.seek(0)
-        cwd = os.getcwd()
+    if batch or confirm("OK to modify your requirements files?"):
         with open(cwd + '/requirements-install.txt', 'w') as f:
-            f.write("\n".join(set(temp_f.read().split())))
+            f.write("# this file was generated by `inv install`\n")
+            f.write("\n".join(reqs))
         with open(cwd + '/requirements.txt', 'r+') as f:
             content = f.read()
             include = '-r requirements-install.txt'
@@ -867,23 +984,27 @@ def configure(ctx, list=False, list_only=False):
                 f.seek(0)
                 f.write(content)
 
-    def do_install():
-        cmd = 'manage.py install --noinput'
-        run_in_demo_projects(ctx, cmd, bare=True)
-
-    if list_only:
-        do_listing()
-    else:
-        if list:
-            do_listing()
-        do_install()
+    cmd = sys.executable + " -m pip install --upgrade pip {}".format(
+        ' '.join(reqs))
+    if batch or confirm("{} (y/n) ?".format(cmd)):
+        ctx.run(cmd)
 
 
-@task(name='prep')
-def prep(ctx, cov=False):
+@task(
+    name='prep',
+    help=dict(
+        after=
+        "Run demo_prep_command only in the demo projects *after* the named one.",
+        start=
+        "Run demo_prep_command only in the demo projects *starting at* the named one."
+    ))
+def prep(ctx, cov=False, after=None, start=None):
     """
-    Run preparation tasks that need to run before testing, but only once for all
-    tests.
+    Run preparation tasks that need to run before testing.
+
+    See also the `prep_command` and `demo_prep_command` in your `tasks.py` file.
+
+    See https://atelier.lino-framework.org/invlib.html#command-inv-prep
 
     """
     # if cov:
@@ -901,7 +1022,9 @@ def prep(ctx, cov=False):
         ctx.run(cmd, pty=True)
     cmd = ctx.demo_prep_command
     if cmd:
-        run_in_demo_projects(ctx, cmd, cov=cov)
+        # run_in_demo_projects(ctx, cmd, cov=cov)
+        run_in_demo_projects(ctx, cmd, bare=True, after=after, start=start)
+        # run_in_demo_projects(ctx, cmd)
 
 
 # @task(name='cov', pre=[run_tests])
@@ -909,7 +1032,7 @@ def prep(ctx, cov=False):
 def run_tests_coverage(ctx, html=True, html_cov_dir='htmlcov'):
     """Run all tests and create a coverage report.
 
-    If there a directory named :xfile:`htmlcov` in your project's
+    If there is a directory named :xfile:`htmlcov` in your project's
     `root_dir`, then it will write a html report into this directory
     (overwriting any files without confirmation).
 
@@ -946,8 +1069,8 @@ def run_tests_coverage(ctx, html=True, html_cov_dir='htmlcov'):
     ctx.run('coverage erase', pty=True)
     print("Running {0} in {1} within coverage...".format(
         ctx.coverage_command, ctx.project_name))
-    ctx.run('coverage run --parallel-mode {}'.format(
-        ctx.coverage_command), pty=True)
+    ctx.run('coverage run --parallel-mode {}'.format(ctx.coverage_command),
+            pty=True)
     if False:
         ctx.run('coverage combine', pty=True)
         ctx.run('coverage report', pty=True)
