@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2013-2016 by Rumma & Ko Ltd.
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """This is an extension for :mod:`atelier.fablib` for managing Java
 projects.
 
@@ -66,8 +65,7 @@ file.
         self.sources = list(self.sourcedir.listdir('*.java'))
 
         self.jarcontent = [Path('Manifest.txt')]
-        self.jarcontent += [
-            Path(x) for x in self.sourcedir.listdir('*.class')]
+        self.jarcontent += [Path(x) for x in self.sourcedir.listdir('*.class')]
 
     def add_lib(self, pth):
         self.libjars.append(Path(pth))
@@ -85,7 +83,8 @@ file.
         jarfile = outdir / self.jarfile
         if jarfile.needs_update(self.jarcontent):
             jarcontent = [x.replace("$", r"\$") for x in self.jarcontent]
-            ctx.run("jar cvfm %s %s" % (jarfile, ' '.join(jarcontent)), pty=True)
+            ctx.run("jar cvfm %s %s" % (jarfile, ' '.join(jarcontent)),
+                    pty=True)
         run_signer(jarfile)
         for libfile in self.libjars:
             jarfile = outdir / libfile.name
@@ -100,5 +99,3 @@ file.
             flags += " -classpath %s" % cp
         for src in self.sources:
             ctx.run("javac %s %s" % (flags, src), pty=True)
-
-
