@@ -2,7 +2,6 @@
 # doctest atelier/sheller.py
 # Copyright 2016-2021 by Rumma & Ko Ltd.
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """Defines the :class:`Sheller` class for testing bash commands in a doctest.
 
 Others have invented similar things before.
@@ -41,9 +40,9 @@ Usage example:
 projects.py
 test.py
 jarbuilder.py
-setup_info.py
 sheller.py
-doctest_utf8.py
+setup_info.py
+sphinxconf.py
 utils.py
 __init__.py
 
@@ -51,6 +50,7 @@ In case you wonder: We sort the by size because that's reproducible. Seems that
 some locales put underscores before letters while others don't.
 
     """
+
     def __init__(self, cwd=None):
         if cwd is None:
             self.temp_dir = TemporaryDirectory()
@@ -75,10 +75,12 @@ some locales put underscores before letters while others don't.
         if self.cwd:
             kwargs.update(cwd=self.cwd)
 
-        process = subprocess.Popen(
-            cmd, stdout=subprocess.PIPE,
-            universal_newlines=True,
-            stderr=subprocess.STDOUT, shell=True, **kwargs)
+        process = subprocess.Popen(cmd,
+                                   stdout=subprocess.PIPE,
+                                   universal_newlines=True,
+                                   stderr=subprocess.STDOUT,
+                                   shell=True,
+                                   **kwargs)
         output, unused_err = process.communicate()
         retcode = process.poll()
         output = output.strip()
