@@ -1,6 +1,5 @@
-# Copyright 2013-2021 Rumma & Ko Ltd.
+# Copyright 2013-2022 Rumma & Ko Ltd.
 # License: GNU Affero General Public License v3 (see file COPYING for details)
-
 """
 Defines tools for testing.
 
@@ -21,7 +20,7 @@ import subprocess
 
 from atelier.utils import SubProcessParent
 
-DOCTEST_CMD = "doctest"
+DOCTEST_CMD = ["doctest", "-o", "REPORT_ONLY_FIRST_FAILURE"]
 
 
 def interpreter_args():
@@ -36,12 +35,14 @@ class DocTestCase(unittest.FunctionTestCase, SubProcessParent):
 
     def __init__(self, filename, addenv=None):
         self.addenv = addenv
+
         def func():
             args = [sys.executable]
             args += ["-m"]
-            args += [DOCTEST_CMD]
+            args += DOCTEST_CMD
             args += [filename]
             self.run_subprocess(args)
+
         func.__name__ = filename
         super(DocTestCase, self).__init__(func)
 
@@ -53,11 +54,10 @@ class DocTestCase(unittest.FunctionTestCase, SubProcessParent):
         return env
 
 
-def make_docs_suite(docs_root, include="*.rst", exclude=None,
-                    addenv=None):
+def make_docs_suite(docs_root, include="*.rst", exclude=None, addenv=None):
     """
     Discover the doc files in specified directory docs_root and below
-    and return a test suite which tests them all, each one in a
+    and return a test suite that tests them all, each one in a
     separate subprocess.
 
     `include` is a filename pattern of the files to include. Default
@@ -89,7 +89,6 @@ def make_docs_suite(docs_root, include="*.rst", exclude=None,
                 return True
         return False
 
-
     for root, dirs, files in os.walk(docs_root):
         dirs.sort()
         for file in sorted(files):
@@ -108,6 +107,7 @@ class TestCase(unittest.TestCase, SubProcessParent):
     "A unittest testcase with some additional utility methods."
 
     project_root = NotImplementedError
+
     # maxDiff = None
 
     def run_packages_test(self, declared_packages):
@@ -139,7 +139,7 @@ class TestCase(unittest.TestCase, SubProcessParent):
                     args = interpreter_args()
                     args += ["-m"]
                     # args += ["doctest"]
-                    args += [DOCTEST_CMD]
+                    args += DOCTEST_CMD
                     args += [fn]
                     self.run_subprocess(args, **kw)
                 if not ok:
